/**
 * 监听一个或多个元素是否出现在视图中
 * @param {HTMLElement | HTMLElement[]} elements  dom元素,多个传入数组
 * @param {IntersectionObserverInit} options IntersectionObserver配置参数
 * @param {function} callback dom变化回调函数
 * @returns {Object} unobserveElement 单个移除某个dom的监听事件
 * @returns {Object} unobserveAllElements 停止观察所有元素
 * @returns {Object} disconnectObserver 关闭观察器
 */
export function useMultiIntersectionObserver(elements, callback, options = {}) {

    // 验证参数
    if (!elements || (Array.isArray(elements) && elements.length === 0)) {
        throw new Error('参数 elements 不能为空');
    }

    // 判断是否为单个元素
    const isSingleElement = !Array.isArray(elements);
    const targetElements = isSingleElement ? [elements] : elements;

    /**
     * 创造IntersectionObserver实例对象
     */
    const observer = new IntersectionObserver((entries, observer) => {
        callback && callback(entries, observer);
    }, options);

    // 开始监听 DOM 变化
    targetElements.forEach((element) => {
        observer.observe(element);
    });

    /**
     * 停止观察某个元素
     * @param {HTMLElement} element 要停止观察的 DOM 元素
     */
    const unobserveElement = (element) => {
        if (element) {
            observer.unobserve(element);
        }
    };

    /**
     * 停止观察所有元素
     */
    const unobserveAllElements = () => {
        for (const element of targetElements) {
            observer.unobserve(element);
        }
    };

    /**
     * 关闭观察器
     */
    const disconnectObserver = () => {
        observer.disconnect();
    };

    return {
        unobserveElement,
        unobserveAllElements,
        disconnectObserver
    };
}
